#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
#include <functional>

using namespace std;
using ll = long long;
using ld = long double;

const ll mod = 1000000007;

struct Tree {
    int num_nodes{};
    int next_edge_id { 0 };
    int num_edge_components{};
    vector<int> depths{};
    vector<int> parents{};
    vector<int> parent_edge_ids{};
    vector<int> jumps{};
    vector<vector<pair<int, int>>> edges{};
    vector<pair<int, int>> edge_list{};
    vector<int> edge_dsu{};
    vector<int> edge_dsu_min_depth_edges{};
    vector<pair<int, int>> opposite_direction_edge_pairs{};

    Tree(int num_nodes) : num_nodes(num_nodes) {
        num_edge_components = num_nodes - 1;
        depths.resize(num_nodes);
        parents.resize(num_nodes);
        parent_edge_ids.resize(num_nodes);
        jumps.resize(num_nodes);
        edges.resize(num_nodes);
        edge_list.resize(num_nodes - 1);
        edge_dsu.assign(num_nodes - 1, -1);
        edge_dsu_min_depth_edges.resize(num_nodes - 1);
    }

    void add_edge(int node1, int node2) {
        edges[node1].push_back({ node2, next_edge_id });
        edges[node2].push_back({ node1, next_edge_id });
        next_edge_id++;
    }

    void init(int root = 0) {
        do_init(root, -1, -1, 0);
    }

    void do_init(int node, int parent, int parent_edge_id, int depth) {
        parents[node] = parent;
        parent_edge_ids[node] = parent_edge_id;
        depths[node] = depth;

        if (parent == -1) {
            jumps[node] = node;
        } else {
            if (depths[jumps[parent]] - depths[parent] == depths[jumps[jumps[parent]]] - depths[jumps[parent]]) {
                jumps[node] = jumps[jumps[parent]];
            } else {
                jumps[node] = parent;
            }
        }

        for (auto [to, id] : edges[node]) {
            if (to == parent) {
                continue;
            }

            edge_dsu_min_depth_edges[id] = id;
            edge_list[id] = { node, to };

            do_init(to, node, id, depth + 1);
        }
    }

    int get_kth_parent(int node, int parent_order) {
        int target_depth = depths[node] - parent_order;
        int current = node;

        while (depths[current] > target_depth) {
            if (depths[jumps[current]] < target_depth) {
                current = parents[current];
            } else {
                current = jumps[current];
            }
        }

        return current;
    }

    int get_lca(int node1, int node2) {
        if (depths[node1] < depths[node2]) {
            swap(node1, node2);
        }

        node1 = get_kth_parent(node1, depths[node1] - depths[node2]);

        while (node1 != node2) {
            if (jumps[node1] == jumps[node2]) {
                node1 = parents[node1];
                node2 = parents[node2];
            } else {
                node1 = jumps[node1];
                node2 = jumps[node2];
            }
        }

        return node1;
    }

    int get_edge_leader(int edge_id) {
        if (edge_dsu[edge_id] < 0) {
            return edge_id;
        }

        return edge_dsu[edge_id] = get_edge_leader(edge_dsu[edge_id]);
    }

    int get_edge_depth(int edge_id) {
        return depths[edge_list[edge_id].first];
    }

    void join_adjacent_edges(int edge_id1, int edge_id2) {
        int leader1 = get_edge_leader(edge_id1);
        int leader2 = get_edge_leader(edge_id2);

        if (leader1 == leader2) {
            return;
        }

        int rank1 = -edge_dsu[leader1];
        int rank2 = -edge_dsu[leader2];

        if (rank1 < rank2) {
            swap(leader1, leader2);
        } else {
            edge_dsu[leader1] -= 1;
        }

        edge_dsu[leader2] = leader1;

        if (get_edge_depth(edge_dsu_min_depth_edges[leader2]) < get_edge_depth(edge_dsu_min_depth_edges[leader1])) {
            edge_dsu_min_depth_edges[leader1] = edge_dsu_min_depth_edges[leader2];
        }

        num_edge_components--;
    }

    void join_path(int child, int parent) {
        int curr_edge_id = parent_edge_ids[child];

        if (curr_edge_id < 0) {
            return;
        }

        while (get_edge_depth(curr_edge_id) > depths[parent]) {
            int next_edge_id = parent_edge_ids[edge_list[edge_dsu_min_depth_edges[get_edge_leader(curr_edge_id)]].first];

            if (next_edge_id < 0 || get_edge_depth(next_edge_id) < depths[parent]) {
                break;
            }

            join_adjacent_edges(curr_edge_id, next_edge_id);
            curr_edge_id = next_edge_id;
        }
    }

    void join_nodes(int node1, int node2) {
        int lca = get_lca(node1, node2);

        join_path(node1, lca);
        join_path(node2, lca);

        if (node1 == lca || node2 == lca) {
            return;
        }

        node1 = get_kth_parent(node1, depths[node1] - depths[lca] - 1);
        node2 = get_kth_parent(node2, depths[node2] - depths[lca] - 1);

        opposite_direction_edge_pairs.push_back({ parent_edge_ids[node1], parent_edge_ids[node2] });
    }
};

bool is_bipartite(const vector<vector<int>>& edges) {
    vector<int> colors(edges.size());

    function<bool(int)> dfs = [&](int start) -> bool {
        for (int to : edges[start]) {
            if (colors[to] == colors[start]) {
                return false;
            } else if (colors[to] == 0) {
                colors[to] = -colors[start];
                
                if (!dfs(to)) {
                    return false;
                }
            }
        }

        return true;
    };

    for (int i = 0; i < edges.size(); i++) {
        if (colors[i] != 0) {
            continue;
        }

        colors[i] = 1;
        
        if (!dfs(i)) {
            return false;
        }
    }

    return true;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int num_nodes, num_pairs;
    cin >> num_nodes >> num_pairs;

    Tree tree(num_nodes);

    for (int i = 0; i < num_nodes - 1; i++) {
        int node1, node2;
        cin >> node1 >> node2;

        tree.add_edge(node1 - 1, node2 - 1);
    }

    tree.init();

    for (int i = 0; i < num_pairs; i++) {
        int node1, node2;
        cin >> node1 >> node2;

        tree.join_nodes(node1 - 1, node2 - 1);
    }

    vector<vector<int>> component_edges(num_nodes - 1);

    for (auto [edge1, edge2] : tree.opposite_direction_edge_pairs) {
        int leader1 = tree.get_edge_leader(edge1);
        int leader2 = tree.get_edge_leader(edge2);

        component_edges[leader1].push_back(leader2);
        component_edges[leader2].push_back(leader1);
    }

    // if (!is_bipartite(component_edges)) {
    //     cout << "0\n";
    //     return 0;
    // }

    ll answer = 1;

    for (int i = 0; i < tree.num_edge_components; i++) {
        answer = (answer * 2) % mod;
    }

    cout << answer << "\n";

    return 0;
}
